/**
 * 
 */
package chat.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;

public class Connection implements Runnable {

    // socket de communication avec le client qui a ouvert cette connexion
    private Socket socket;
    // flot de sortie associé au socket
    private PrintStream outputStream;
    // flot d'entrée associé au socket
    private BufferedReader inputStream;
    // référence à l'instance du serveur sur lequel cette connexion a été ouverte
    private Server server;
    // état de la connexion
    private volatile boolean running = false;
    //Alias du client
    private String alias=null;

    /**
     * Constructeur
     * @param socket socket de communication avec le client
     * @param server instance du serveur sur lequel cette connexion a été ouverte
     * @throws IOException
     */
    public Connection(Socket socket, Server server) throws IOException {
        this.socket = socket;
        this.outputStream = new PrintStream(socket.getOutputStream());
        this.inputStream = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        this.server = server;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    @Override
    public void run() {
        try {
            running = true;
            while (running) {
                try {
                    String message = inputStream.readLine();
                    if (message != null) {
                        System.out.println("log : " + message);
                        processMessage(message);
                    }
                }
                catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        finally {
            try {
                server.removeConnection(this);
                socket.close();
            }
            catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

    /**
     * Méthode qui traite un message reçu du client
     * @param message message à traiter
     */
    private void processMessage(String message) {
        // si le message se termine par #QUIT, fin de la connexion
        if (message.endsWith("#QUIT")) {
            running = false;
        }
        //Sinon si le message se termine par #ALIAS, definit l'alias du connecté.
        else if(message.endsWith("#ALIAS")){
        	this.alias = message.substring(0, message.length()-6);
        }
        // sinon on diffuse le message à tous les clients
        else
            server.broadcastMessage(/*this.alias+" > "+*/message);
    }

    /**
     * Ascesseur sur l'alias du client connecté
     * @return l'alias du client.
     */
    public String getAlias(){
    	return this.alias;
    }
    
    /**
     * Envoi d'un message au client qui a ouvert cette connexion
     * @param message message à envoyer
     */
    public void sendMessage(String message) {
        outputStream.println(message);
    }

}
