/**
 * 
 */
package chat.server;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class Server {
    // Le socket en attente de demande de connexion entrante
    private ServerSocket serverSocket;
    // Liste des connexions actives
    private ArrayList<Connection> connections;
    // Pool de gestion des threads du serveur
    private ExecutorService pool;
    //Liste des alias des client
    private ArrayList<String> alias;

    /**
     * Constructeur du serveur
     * 
     * @param port port d'écoute
     * @throws IOException si le socket d'écoute ne peut pas être ouvert
     */
    public Server(int port) throws IOException {
        serverSocket = new ServerSocket(port);
        System.out.println("Serveur en écoute sur le port : " + port);
        connections = new ArrayList<>();
        pool = Executors.newCachedThreadPool();
        alias = new ArrayList<String>();
    }

    /**
     * démarrage du serveur
     */
    public void start() {
        pool.submit(new Runnable() {
			
			@Override
			public void run() {
		            while (true) {
		                Socket client = null;
		                Connection connection = null;
						try {
							client = serverSocket.accept();
							System.out.println("Demande de connexion...");
		           			connection = new Connection(client, Server.this);
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
		                synchronized (connections) {
		                	//if (aliasValide(connection.getAlias()))
		                		connections.add(connection);
		                }
		                pool.submit(connection);
		            }
			}
		});
    }
    /**
     * Diffusion d'un message à tous les clients connectés
     * @param message message à diffuser
     */
    public void broadcastMessage(String message) {
        synchronized (connections) {
            for (Connection connection : connections)
                connection.sendMessage(message);
        }
    }

    /**
     * Suppression d'une connexion
     * @param connection connexion à supprimer
     */
    public void removeConnection(Connection connection) {
        synchronized (connections) {
            connections.remove(connection);
        }
    }

    /**
     * Vérifie si l'alias du client n'est pas déjà utilsé.
     * @param nomClient alias à vérifier.
     * @return true si l'alias n'hésisté pas, sinon false
     
    public boolean aliasValide(String nomClient) {
    	boolean res = true;
    	for (String alias : this.alias) {
    		if (alias.equalsIgnoreCase(nomClient))res =false;
    	}
    	if (res)this.alias.add(nomClient);
    	return res;
    }*/
    
    public static void main(String[] args) {
        try {
            Server server = new Server(3333);
            server.start();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
}
